package com.ninexiu.sixninexiu.audio

import android.media.MediaPlayer
import android.text.TextUtils
import com.ninexiu.sixninexiu.common.util.NSLog
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.schedulers.Schedulers
import java.lang.Exception
import java.util.concurrent.TimeUnit

/**
 * <pre>
 *     @author : hpf
 *     time   : 2021/03/30
 *     desc   : 音频播放管理类
 *     version: 1.0
 * </pre>
 */
object AudioPlayerManager {
    private var listener: OnAudioPlayerListener? = null
    private var mediaPlayer : MediaPlayer? = null
    private var currentPlayPath: String? = null
    private var duration: Int = 0

    fun startPlayAudioPause(path: String, duration: Int, listener: OnAudioPlayerListener? = null) {
        // 试听的暂停功能
        if(TextUtils.equals(currentPlayPath,path)) {
            releasePlayer()
            return
        }
        startPlayAudio(path, duration, listener)
    }

    // 播放音频文件
        fun startPlayAudio(path: String, duration: Int, listener: OnAudioPlayerListener? = null) {
        if(TextUtils.equals(currentPlayPath,path)) {
            return
        }
        try {

            // 流程 : 1. 先去释放以前的音频播放资源，监听置为空
            releasePlayer()

            currentPlayPath = path
            this.duration = duration
            this.listener = listener

            mediaPlayer = MediaPlayer()

            // 流程 : 2. 设置新的播放资源
            mediaPlayer?.setDataSource(path)
            mediaPlayer?.setOnCompletionListener {
                // 流程 : 5. 播放结束 -> 正常结束
                playEndOrFail(false)
            }
            mediaPlayer?.setOnErrorListener { mp, what, extra ->
                NSLog.d(AudioRecordManager.Tag, "error : $what , $extra")
                // 流程 : 5. 播放结束 -> 播放出错
                playEndOrFail(true)
                true
            }

            mediaPlayer?.setVolume(1f, 1f)
            mediaPlayer?.isLooping = false

            mediaPlayer?.setOnPreparedListener {
                currentPlayPath?.let {
                    // 流程 : 3. 播放开始
                    listener?.onAudioPlayStart(it)
                }
                // 流程 : 4. 播放倒计时回调
                playCutDown(duration)
                mediaPlayer?.start()
            }
            mediaPlayer?.prepareAsync()
        } catch (e: Exception) {
            NSLog.d(AudioRecordManager.Tag, e.message)
            playEndOrFail(true)
        }
    }

    /**
     * 播放倒计时
     */
    private fun playCutDown(duration: Int) {
        if (duration <= 0) {
            return
        }

        var durationTemp = duration * 1000L
        Observable.interval(500, TimeUnit.MILLISECONDS)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    durationTemp -= 500
                    if (durationTemp >= 0) {
                        currentPlayPath?.let {
                            listener?.onAudoPlayCutDown(it, (durationTemp / 1000).toInt())
                        }
                    }
                }.also { compositeDisposable.add(it) }
    }


    /**
     * 播放结束或播放出错
     */
    private fun playEndOrFail(error: Boolean) {
        currentPlayPath?.let {
            listener?.onAudioPlayEnd(it, error)
        }
        releasePlayer()     // 结束或出错，释放资源
    }


    private val compositeDisposable = CompositeDisposable()

    /**
     * 释放音频文件
     */
    fun releasePlayer() {
        // 流程 : 6. 将音频资源释放,外界需要做一些动画操作
        currentPlayPath?.let { listener?.onAudioPlayEnd(it, false) }

        currentPlayPath = null
        listener = null
        mediaPlayer?.let {
            with(it) {
                setOnCompletionListener(null)
                setOnErrorListener(null)
                stop()
                release()
            }
        }
        mediaPlayer = null
        compositeDisposable.clear()
    }

}

interface OnAudioPlayerListener {
    // 播放开始
    fun onAudioPlayStart(path: String)

    // 播放过程回调
    fun onAudoPlayCutDown(path: String, duration: Int)

    // 播放音频结束
    fun onAudioPlayEnd(path: String, isError: Boolean)
}