package com.ninexiu.sixninexiu.audio

import android.media.MediaPlayer
import android.media.MediaRecorder
import android.os.Build
import androidx.annotation.RequiresApi
import com.ninexiu.sixninexiu.common.util.NSLog
import com.zhongrun.nineshow_base.utils.NineShowFilePathManager
import io.reactivex.Flowable
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.schedulers.Schedulers
import java.io.File
import java.util.concurrent.TimeUnit

/**
 * <pre>
 *     @author : hpf
 *     time   : 2021/03/19
 *     desc   :
 *     version: 1.0
 * </pre>
 */
class AudioRecordManager(private val minDuration: Int = 0, private val maxDuration: Int = 60, val listener: OnAudioRecordListener? = null) {
    companion object {
        const val Tag = "AudioRecordManager >> "
    }

    private val compositeDisposable = CompositeDisposable()
    private var mediaRecorder: MediaRecorder? = null

    /**
     * 开始录音
     */
    fun startRecording() {
        // 先释放资源
        releaseRecord()
        // 开始录音
        startRecordReal()
    }

    /**
     * 开始录音
     */
    // 录音目录
    private val audioParentDir = NineShowFilePathManager.INSTANCE.getExterFileDir(NineShowFilePathManager.AUDIO_RECORD_DIR)
    private var audioFile: File? = null

    private fun startRecordReal() {
        mediaRecorder = MediaRecorder()
        // 创建录音文件，.m4a为MPEG-4音频标准的文件的扩展名
        audioFile = File("${audioParentDir}${File.separator}${System.currentTimeMillis()}.m4a")
        audioFile?.parentFile?.mkdirs()

        try {
            audioFile?.createNewFile()

            // 设置MediaRecorder参数
            mediaRecorder?.let {
                with(it) {
                    // 从麦克风采集声音数据
                    setAudioSource(MediaRecorder.AudioSource.MIC)
                    // 这里设置为 AAC_ADTS 格式，是为了ios那边能正常使用
                    setOutputFormat(MediaRecorder.OutputFormat.AAC_ADTS)
                    // 设置采样频率,44100是所有安卓设备都支持的频率,频率越高，音质越好，当然文件越大
                    setAudioSamplingRate(44100)
                    // 设置声音数据编码格式,音频通用格式是AAC
                    setAudioEncoder(MediaRecorder.AudioEncoder.AAC)
                    // 设置编码频率
                    setAudioEncodingBitRate(96000)
                    setOutputFile(audioFile?.absolutePath)
                    prepare()
                    start()
                    // 录音开始计时
                    audioTimeCutDown()
                }
            }
        } catch (e: Exception) {
            NSLog.d(Tag, e.message)
            releaseRecord()
            // 初始化失败
            listener?.onAudioInitFailed(e.message ?: "error")
        }
    }

    /**
     * 录音开始计时
     */
    var duration: Int = 0
    private fun audioTimeCutDown() {
        listener?.onAudioRecordStart()  // 回调
        val startTime = System.currentTimeMillis()

        Observable.interval(500, TimeUnit.MILLISECONDS)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    duration = ((System.currentTimeMillis() - startTime) / 1000).toInt()

                    listener?.onAudioProgress(duration)

                    if (duration >= maxDuration) {
                        // 录音结束
                        stopAudioRecord()
                    }
                }.also { compositeDisposable.add(it) }
    }


    /**
     * 停止录音
     */
    fun stopAudioRecord() {
        releaseRecord()
        if (duration >= minDuration && duration <= (maxDuration + 1)) {
            listener?.onAudioFinish(true, audioFile?.absolutePath, duration)
        } else { // 时长未满足，无效录音
            listener?.onAudioFinish(false, null, -1)
            audioFile?.absolutePath?.let { deleteAudioFile(it) }

        }
        duration = -1
    }

    /**
     * 删除无效资源
     */
    fun deleteAudioFile(filePath : String) {
        val file = File(filePath)
        if (file.exists() && file.isFile) {
            NSLog.d(Tag,"删除成功...")
            file.delete()
        }
    }

    /**
     * 释放录音资源
     */
    fun releaseRecord() {
        // mediaRecorder?.pause()
        mediaRecorder?.release()
        mediaRecorder = null
        compositeDisposable.clear()
    }
}

