package com.ninexiu.sixninexiu.audio

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.animation.Animation
import android.view.animation.ScaleAnimation
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.fragment.app.FragmentActivity
import com.ninexiu.sixninexiu.R
import com.ninexiu.sixninexiu.common.util.MyToast
import com.ninexiu.sixninexiu.common.util.NSLog
import com.ninexiu.sixninexiu.common.util.manager.PermissionManager
import kotlinx.android.synthetic.main.view_audio_item.view.*
import kotlinx.android.synthetic.main.view_audio_record.view.*
import java.io.File

/**
 * <pre>
 *     @author : hpf
 *     time   : 2021/03/18
 *     desc   :
 *     version: 1.0
 * </pre>
 */
class AudioRecordView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) : ConstraintLayout(context, attrs, defStyleAttr) {
    companion object {
        const val Tag = "AudioRecordView >> "
        const val STATUS_INIT = 0       // 录音初始化状态
        const val STATUS_RECORDING = 1  // 录音中状态
        const val STATUS_END = 2        // 录音结束状态
        const val STATUS_LISTENING = 3  // 录音试听
        const val MIN_DURATION = 3
        const val MAX_DURATION = 60
    }

    private var status = STATUS_INIT

    init {
        LayoutInflater.from(context).inflate(R.layout.view_audio_record, this)


        audioRlStatusChange.setOnClickListener {
            // 切换状态
            switchAudioStatus()
        }

        audioIvCancel.setOnClickListener {
            // 删除音频文件
            audioRecordFilePath?.let {
                AudioPlayerManager.releasePlayer()
                recordManager.deleteAudioFile(it)
            }
            setStatus(STATUS_INIT)
        }

        audioIvConfirm.setOnClickListener {
            NSLog.d(Tag, audioRecordFilePath ?: "file is null")
            if (audioRecordFilePath.isNullOrEmpty() || audioDuration <= 0) {
                MyToast.showShort("录音文件出错")
            } else {
                audioRecordFilePath?.let {
                    audioComplete?.invoke(it, audioDuration)        // 录音文件完成
                }
            }
            AudioPlayerManager.releasePlayer()
            setStatus(STATUS_INIT)
        }
    }

    var audioComplete: ((String, Int) -> Unit)? = null

    private var audioDuration = -1

    // 录音管理类的回调
    private val recordListener: OnAudioRecordListener = object : OnAudioRecordListener {
        override fun onAudioInitFailed(msg: String) {
            // 流程 : error. 若是MediaRecorder初始化失败，则ui恢复初始状态
            setStatus(STATUS_INIT)
        }

        override fun onAudioRecordStart() {
            // 流程 : 2. 录音由初始化状态变为录音中状态
            if (status == STATUS_INIT) {
                setStatus(STATUS_RECORDING)
            }
        }

        @SuppressLint("SetTextI18n")
        override fun onAudioProgress(duration: Int) {
            // 流程 : 3. 录音中的录音时长回调
            if (status == STATUS_RECORDING) {
                audioTvTime.text = "${duration}s"
            }
        }

        override fun onAudioFinish(effective: Boolean, audioFilePath: String?, duration: Int) {
            // 流程 : 5. 录音结束，判断时长是否满足，不满足恢复初始状态，满足进入录音结束状态
            if (!effective) {
                setStatus(STATUS_INIT)
                MyToast.showShort(context.getString(R.string.audio_record_time_short))
                return
            }

            audioDuration = duration
            audioRecordFilePath = audioFilePath

            startIvScaleAnimation {      // todo  : 动画结束以后显示
                setStatus(STATUS_END)
            }
        }
    }

    // 播放音频的管理类监听
    private val onAudioPlayerListener = object : OnAudioPlayerListener {
        override fun onAudioPlayStart(path: String) {
            // 流程 : 7. 开始播放录音，更改ui状态
            setStatus(STATUS_LISTENING)
        }

        override fun onAudoPlayCutDown(path: String, duration: Int) {
            // 流程 : 8. 播放倒计时
            audioTvTime.text = "${duration}s"
        }

        @SuppressLint("SetTextI18n")
        override fun onAudioPlayEnd(path: String, isError: Boolean) {
            if (isError) {
                MyToast.showShort(context.getString(R.string.audio_play_error))
            }

            setStatus(STATUS_END)
        }
    }

    private var audioRecordFilePath: String? = null
    private val recordManager = AudioRecordManager(MIN_DURATION, MAX_DURATION, recordListener)

    // 用户状态的切换 -> 切换录音不同的状态
    private fun switchAudioStatus() {
        when (status) {
            STATUS_RECORDING -> {
                // 流程 : 4. 用户主动结束录音，这时候需要判断录音时长
                recordManager.stopAudioRecord()
            }

            STATUS_END -> {
                // 流程 : 6. 用户试听录音
                audioRecordFilePath?.let { AudioPlayerManager.startPlayAudio(it, audioDuration, onAudioPlayerListener) }
                // audioRecordFilePath?.let { recordManager.startPlayAudio(it, audioDuration) }
            }

            // 流程 : 1.用户在初始状态点击开始录音，初始化MediaRecorder
            STATUS_INIT -> {
                if (activity !is FragmentActivity) {
                    MyToast.showShort(context.getString(R.string.audio_record_error))
                    return
                }

                // 先判断是否有录音权限
                (activity as? FragmentActivity)?.let {
                    PermissionManager.create().showAudio(it) {
                        recordManager.startRecording()
                    }
                }
            }
            STATUS_LISTENING -> {
                AudioPlayerManager.releasePlayer()
                setStatus(STATUS_END)
            }
        }
    }

    private var activity: Activity? = null
    fun setActivity(activity: Activity) {
        this.activity = activity
    }

    // 用户状态直接设置 -> 可以直接设置某种状态
    @SuppressLint("UseCompatLoadingForDrawables", "SetTextI18n")
    fun setStatus(status: Int) {
        this.status = status        // 全局状态
        onAudioStatusChanged?.invoke(status)
        when (status) {
            STATUS_RECORDING -> {
                startWaveAnimation(false)
                audioRlStatusNB.visibility = View.INVISIBLE
                audioRlStatusRecording.visibility = View.VISIBLE
                audioTvInfo.text = "录音中"
                audioTvInfo.background = resources.getDrawable(R.drawable.shape_audio_f2f2f2)
                audioTvTime.visibility = View.VISIBLE
                audioProgress.startProgress()
            }

            STATUS_END -> {
                audioTvInfo.text = "已完成,点击试听"
                audioTvTime.text = "${audioDuration}s"
                audioTvInfo.background = resources.getDrawable(R.drawable.shape_audio_f2f2f2)

                audioIvCancel.visibility = View.VISIBLE
                audioIvConfirm.visibility = View.VISIBLE

                audioRlStatusNB.visibility = View.VISIBLE
                audioIvDot.visibility = View.INVISIBLE
                audioVoiceWave.visibility = View.VISIBLE

                audioRlStatusRecording.visibility = View.INVISIBLE
                audioProgress.resetProgress()               // 停止进度的动画

                startWaveAnimation(false)
                audioVoiceWave.stopWave()
            }

            STATUS_LISTENING -> {
                audioTvInfo.text = "试听中"
                // 开始波浪动画
                startWaveAnimation(true)
                audioVoiceWave.startWave()
            }

            STATUS_INIT -> {
                audioDuration = -1
                audioRecordFilePath = null
                audioTvInfo.text = "点击按钮即可录制语音，每次不超过60s"
                audioTvTime.text = "0s"
                audioTvInfo.background = null
                audioTvTime.visibility = View.INVISIBLE

                // 动画的view
                audioIvCancel.visibility = View.INVISIBLE
                audioIvConfirm.visibility = View.INVISIBLE

                // 开始波浪动画
                startWaveAnimation(true)

                // 初始化状态可见view
                audioRlStatusNB.visibility = View.VISIBLE
                audioIvDot.visibility = View.VISIBLE
                audioVoiceWave.visibility = View.INVISIBLE
                audioVoiceWave.stopWave()               // 停止语音的动画

                audioRlStatusRecording.visibility = View.INVISIBLE
                audioProgress.resetProgress()               // 停止进度的动画
            }
        }
    }

    val scale = 64f / 34
    private fun startIvScaleAnimation(onEnd: (() -> Unit)? = null) {
        val ivRecorderAnimation = ScaleAnimation(1f, scale, 1f, scale, Animation.RELATIVE_TO_SELF, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f)
        ivRecorderAnimation.duration = 500
        ivRecorderAnimation.fillAfter = false
        audioIvRecordingCenter.startAnimation(ivRecorderAnimation)

        ivRecorderAnimation.setAnimationListener(object : Animation.AnimationListener {
            override fun onAnimationRepeat(animation: Animation?) {
            }

            override fun onAnimationEnd(animation: Animation?) {
                onEnd?.invoke()
                audioIvRecordingCenter.clearAnimation()
            }

            override fun onAnimationStart(animation: Animation?) {
            }

        })
    }


    // 控制外层波浪动画
    private var waveScaleAnimation: ScaleAnimation? = null
    private fun startWaveAnimation(start: Boolean) {
        if (start) {
            waveScaleAnimation = ScaleAnimation(1f, 0.8f, 1f, 0.8f, Animation.RELATIVE_TO_SELF, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f)
            waveScaleAnimation?.duration = 1000
            waveScaleAnimation?.fillAfter = false
            waveScaleAnimation?.repeatCount = Animation.INFINITE
            waveScaleAnimation?.repeatMode = Animation.REVERSE
            audioViewWave.startAnimation(waveScaleAnimation)
        } else {
            audioViewWave.clearAnimation()
        }
    }

    // 录音状态切换的回调
    var onAudioStatusChanged : ((Int) -> Unit)? = null


    fun releaseAudioRecorder(){
        // 删除音频文件
        audioRecordFilePath?.let {
            AudioPlayerManager.releasePlayer()
            recordManager.deleteAudioFile(it)
        }
        recordManager.releaseRecord()
        setStatus(STATUS_INIT)
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        releaseAudioRecorder()
    }


}
