package com.ninexiu.sixninexiu.view

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.animation.ObjectAnimator
import android.animation.ValueAnimator
import android.content.Context
import android.graphics.Color
import android.os.Build
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.animation.AccelerateDecelerateInterpolator
import android.widget.LinearLayout
import android.widget.TextView
import androidx.annotation.RequiresApi
import com.ninexiu.sixninexiu.R
import kotlinx.android.synthetic.main.view_input_type.view.*

/**
 * <pre>
 *     @author : hpf
 *     time   : 2020/12/24
 *     desc   : 输入框中选择某个type，伸缩动画
 *     version: 1.0
 * </pre>
 */
class InputTypeAnimatorView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) : LinearLayout(context, attrs, defStyleAttr) {
    companion object {
        const val DURATION = 200L
        const val TYPE_PUBLIC = 0
        const val TYPE_FLY = 1
        const val TYPE_BROADCAST = 2
    }

    /* private val viewWidth :Int
             get() = (context.resources.displayMetrics.density * 51).toInt()*/
    private val viewWidth: Int by lazy {
        (context.resources.displayMetrics.density * 51).toInt()
    }

    var currentType: Int = TYPE_PUBLIC
        private set

    // 记录当前是打开还是收缩的状态
    private var currentStatusShow: Boolean = false

    init {
        LayoutInflater.from(context).inflate(R.layout.view_input_type, this, true)
    }

    private val typeArr = arrayOf("公屏", "飞屏", "广播")
    override fun onFinishInflate() {
        super.onFinishInflate()

        tv1.text = typeArr[0]
        tv2.text = typeArr[1]
        tv3.text = typeArr[2]


        tv1.setOnClickListener {
            showOrDismiss(TYPE_PUBLIC)
        }

        tv2.setOnClickListener {
            showOrDismiss(TYPE_FLY)
        }

        tv3.setOnClickListener {
            showOrDismiss(TYPE_BROADCAST)
        }

        // 初始化View
        initTab()
    }

    // 初始化
    fun initTab(type : Int = TYPE_PUBLIC) {
        currentType = type
        when (currentType) {
            TYPE_PUBLIC -> showView(tv1, tv3, tv2)
            TYPE_FLY -> showView(tv2, tv3, tv1)
            TYPE_BROADCAST -> showView(tv3, tv2, tv1)
        }
        ivArrow.setImageResource(R.drawable.ic_arrow_spread)
        currentStatusShow = false
    }

    private fun showView(viewVisible: TextView, view1: TextView, view2: TextView) {
        setChoseTypeColor()
        setChildLayoutParams(viewVisible, true)
        setChildLayoutParams(view1, false)
        setChildLayoutParams(view2, false)
    }

    private fun setChildLayoutParams(view: View, isShow: Boolean) {
        val par = view.layoutParams
        par.width = if (isShow) viewWidth else 0
        view.layoutParams = par
    }


    // 当type更改时，外界回调
    var choseTypeCallback: ((Int) -> Unit)? = null

    // 选中某一项tab，展开或者消失
    private fun showOrDismiss(choseType: Int) {
        when (currentStatusShow) {
            true -> {
                currentType = choseType
                setChoseTypeColor()
                choseTypeCallback?.invoke(currentType)
                dismiss()
            }
            false -> {
                show()
            }
        }
    }

    // 颜色修改
    private fun setChoseTypeColor(){
            when (currentType) {
                TYPE_PUBLIC -> {
                    tv1.setTextColor(Color.WHITE)
                    tv2.setTextColor(resources.getColor(R.color.color_999999))
                    tv3.setTextColor(resources.getColor(R.color.color_999999))
                }
                TYPE_FLY -> {
                    tv2.setTextColor(Color.WHITE)
                    tv1.setTextColor(resources.getColor(R.color.color_999999))
                    tv3.setTextColor(resources.getColor(R.color.color_999999))
                }
                TYPE_BROADCAST -> {
                    tv3.setTextColor(Color.WHITE)
                    tv1.setTextColor(resources.getColor(R.color.color_999999))
                    tv2.setTextColor(resources.getColor(R.color.color_999999))
                }
            }
    }



    // 隐藏(收缩)
    fun dismiss() {
        when (currentType) {
            TYPE_PUBLIC -> startShrinkAnimation(tv3, tv2)
            TYPE_FLY -> startShrinkAnimation(tv3, tv1)
            TYPE_BROADCAST -> startShrinkAnimation(tv2, tv1)
        }
        currentStatusShow = false
    }

    // 显示(展开)
    private fun show() {
        // setChoseTypeColor()
        when (currentType) {
            TYPE_PUBLIC -> {
                startSpreadAnimation(tv3, tv2)
            }
            TYPE_FLY -> {
                startSpreadAnimation(tv3, tv1)
            }
            TYPE_BROADCAST -> {
                startSpreadAnimation(tv2, tv1)
            }
        }
        currentStatusShow = true
    }

    // 收缩
    private fun startShrinkAnimation(firstView: View, secondView: View) {
        endShrinkAnimation(firstView)
        endShrinkAnimation(secondView)
        doArrowAnimator(false)
    }

    // 展开 : 两种做法 : 1.依次做一个展开动作(体验不是太流畅,加速器未能体现效果)   2.同时进行展开动画
    private fun startSpreadAnimation(firstView: View, secondView: View) {
        endSpreadAnimation(firstView)
        endSpreadAnimation(secondView)
        doArrowAnimator(true)
    }

    // 执行收缩动画，结束后回调
    private fun endShrinkAnimation(view: View, end: (View.() -> Unit)? = null) {
        val valueAnimator = ValueAnimator.ofInt(view.layoutParams.width, 0)
        valueAnimator.addUpdateListener {
            val value = (valueAnimator.animatedValue as? Int) ?: 0

            // 修改View的layoutParams
            val params = view.layoutParams
            params.width = value
            view.layoutParams = params

            if (value == 0) {
                // 动画结束
                end?.invoke(view)
            }
        }

        valueAnimator.interpolator = AccelerateDecelerateInterpolator()
        valueAnimator.duration = DURATION
        valueAnimator.setTarget(view)
        valueAnimator.start()
    }

    // 执行展开动画，结束后回调
    private fun endSpreadAnimation(view: View, end: (View.() -> Unit)? = null) {
        val valueAnimator = ValueAnimator.ofInt(0, viewWidth)
        valueAnimator.addUpdateListener {
            val value = (valueAnimator.animatedValue as? Int) ?: 0

            // 修改View的layoutParams
            val params = view.layoutParams
            params.width = value
            view.layoutParams = params

            if (value == viewWidth) {
                // 动画结束
                end?.invoke(view)
            }
        }

        valueAnimator.interpolator = AccelerateDecelerateInterpolator()
        valueAnimator.duration = DURATION
        valueAnimator.setTarget(view)
        valueAnimator.start()
    }

    // 箭头执行旋转动画
    private fun doArrowAnimator(isSpread: Boolean) {
       /* val arrowAnimator = ObjectAnimator.ofFloat(ivArrow, "rotation", 0.0f, 180f)
        arrowAnimator.addListener(object : AnimatorListenerAdapter() {
            override fun onAnimationEnd(animation: Animator?) {
                ivArrow.clearAnimation()
                if(isSpread) {
                    ivArrow.setImageResource(R.drawable.ic_arrow_spread)
                }else {
                    ivArrow.setImageResource(R.drawable.ic_arrow_shrink)

                }
            }
        })
        arrowAnimator.interpolator = AccelerateDecelerateInterpolator()
        arrowAnimator.duration = 200
        arrowAnimator.start()*/

        if(isSpread) {
            ivArrow.setImageResource(R.drawable.ic_arrow_shrink)
        }else {
            ivArrow.setImageResource(R.drawable.ic_arrow_spread)

        }
    }

}