package com.ninexiu.sixninexiu.view

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.util.AttributeSet
import android.util.Log
import android.view.View
import com.ninexiu.sixninexiu.R

/**
 * <pre>
 *     @author : hpf
 *     time   : 2021/07/14
 *     desc   : 设置宽高无效
 *

使用示例 :
<com.ninexiu.sixninexiu.view.MyWaveView
android:layout_gravity="center_horizontal"
android:id="@+id/wave1"
android:layout_width="wrap_content"
android:layout_height="wrap_content"
app:wave_counts="6"
app:wave_create_duration="800"
app:wave_fill_width="70dp"
app:wave_models="stroke"
app:wave_colors="#FF3366"
app:wave_item_distance="4dp" />

 */
class MyWaveView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) : View(context, attrs, defStyleAttr) {
    companion object {
        const val Tag = "MyWaveView :"
        const val WAVE_REFRESH = 30            // 每次的刷新时间(固定)
    }

    private val paint = Paint()
    private val fillWidth: Float               // 内部的空白区域
    private val waveCreateDuration: Int        // 每次创建一个波浪的时间
    private val waveItemDistance: Float        // 波浪之间的半径差距
    private val waveCount: Int                 // 波浪的个数
    private var sumDuration = 0                // 计算创建波浪时机
    private var waveModel: Paint.Style         // 绘制方式 : 填充或描边
    private var waveColor: Int                 // 波浪颜色
    private val doRepeat: Boolean              // 是否重复

    private val waveCollection = mutableListOf<WaveBean>()

    init {
        paint.isAntiAlias = true
        paint.isDither = true
        paint.strokeCap = Paint.Cap.ROUND
        paint.strokeJoin = Paint.Join.ROUND
        paint.strokeWidth = 2f


        val arr = context.obtainStyledAttributes(attrs, R.styleable.MyWaveView)
        fillWidth = arr.getDimension(R.styleable.MyWaveView_wave_fill_width, 50f)
        waveCreateDuration = arr.getInt(R.styleable.MyWaveView_wave_create_duration, 60)
        waveItemDistance = arr.getDimension(R.styleable.MyWaveView_wave_item_distance, 5f)
        waveCount = arr.getInt(R.styleable.MyWaveView_wave_counts, 3)
        waveColor = arr.getColor(R.styleable.MyWaveView_wave_colors, Color.WHITE)
        waveModel = if (arr.getInt(R.styleable.MyWaveView_wave_models, 0) == 0) Paint.Style.STROKE else Paint.Style.FILL
        doRepeat = arr.getBoolean(R.styleable.MyWaveView_doRepeat, true)

        paint.color = waveColor
        paint.style = waveModel
        arr.recycle()
    }

    // 透明度
    private val waveAlpha: Float by lazy {
        val alpha = Color.alpha(waveColor)
        Log.d(Tag, "alpha : $alpha")
        if (alpha in 0..255) {
            alpha * 1f
        } else {
            0f
        }
    }

    // 每次刷新波浪应该增加的半径   理解 : 波浪每间隔 waveCreateDuration 时间创建一条，而前一条这时候已经扩大了 waveItemDistance
    private val multiplyRadius: Float by lazy {
        waveItemDistance / (waveCreateDuration * 1f / WAVE_REFRESH)
    }

    // 每次刷新应该减少的透明度
    private val reduceAlpha: Float by lazy {
        waveAlpha / ((waveCreateDuration * (waveCount - 1)) / WAVE_REFRESH)
    }

    private val run = Runnable {
        postInvalidate()
    }

    private var start = false

    fun startWave() {
        stopWave()
        start = true
        post(run)
    }

    fun stopWave() {
        start = false
        removeCallbacks(run)
        waveCollection.clear()
        postInvalidate()
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)

        if (!start) {
            return
        }

        if (waveCollection.isEmpty()) {
            // 创建新的波浪
            createWave()
        } else {
            // 改变每个波浪的状态
            for (item in waveCollection) {
                item.radius += multiplyRadius
                item.alpha -= reduceAlpha
            }

            sumDuration += WAVE_REFRESH

            if (sumDuration >= waveCreateDuration) {
                createWave()
            }
        }

        if (waveCollection.isNullOrEmpty()) {
            return
        }

        // FIXME : 测试代码 -> 防止变化突兀
       /* paint.style = Paint.Style.FILL
        paint.alpha = waveAlpha.toInt()
        canvas?.drawCircle(measuredWidth * 1f / 2, measuredHeight * 1f / 2, fillWidth / 2, paint)
        paint.style = waveModel*/

        // 绘制
        waveCollection.forEach { item ->
            paint.alpha = if (item.alpha > 0) item.alpha.toInt() else 0
            canvas?.drawCircle(measuredWidth * 1f / 2, measuredHeight * 1f / 2, item.radius, paint)
        }

        // 延迟刷新
        postDelayed(run, WAVE_REFRESH.toLong())
    }


    private fun createWave(): WaveBean {
        val bean = WaveBean(fillWidth / 2, waveAlpha)
        waveCollection.add(0, bean)
        // 集合中存储的波浪条目有限 : waveCount
        if (waveCollection.size > waveCount) {
            waveCollection.removeAt(waveCount)
        }

        // 重新计数
        sumDuration = 0

        return bean
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        Log.d(Tag, "reduceAlpha : $reduceAlpha")
        val result = (fillWidth + (waveCount) * waveItemDistance * 2 + 0.5f).toInt()
        setMeasuredDimension(result, result)
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        start = false
        waveCollection.clear()
        removeCallbacks(run)
    }
}

data class WaveBean(
        var radius: Float,       // 波纹半径
        var alpha: Float     // 波纹透明度
)

/**
自定义属性 :
<declare-styleable name="MyWaveView">
<attr name="wave_fill_width" format="dimension|reference"/>
<attr name="wave_create_duration" format="integer"/>
<attr name="wave_item_distance" format="dimension|reference"/>
<attr name="wave_counts" format="integer|reference"/>
<attr name="wave_colors" format="color|reference"/>
<attr name="doRepeat" format="boolean"/>
<attr name="wave_models" format="enum">
<enum name="stroke" value="0"/>
<enum name="fill" value="1"/>
</attr>
</declare-styleable>
 */