package com.ninexiu.sixninexiu.view

import android.content.Context
import android.graphics.*
import android.os.Build
import android.text.TextUtils
import android.transition.AutoTransition
import android.transition.TransitionManager
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import com.ninexiu.sixninexiu.BuildConfig
import com.ninexiu.sixninexiu.R
import com.ninexiu.sixninexiu.application.NineShowApplication
import com.ninexiu.sixninexiu.common.util.GlideUtils
import com.ninexiu.sixninexiu.common.util.Utils
import com.opensource.svgaplayer.SVGACallback
import kotlinx.coroutines.*
import org.jetbrains.anko.displayMetrics
import java.net.URL

/**
 * <pre>
 *     @author : hpf
 *     time   : 2020/11/23
 *     desc   : 用于 view 的适配
 *     version: 1.0
 * </pre>
 */

/**
 * 适配 View 的正确摆放位置 , 主要是 margin，不涉及View的自身宽高
 * @view
 * @orientation : 方向，需要匹配的margin为哪个方向
 * @setmargin : 应该设置的 margin 值
 */
const val ORIENTATION_TOP = 0
const val ORIENTATION_LEFT = 1
const val ORIENTATION_RIGHT = 2
const val ORIENTATION_BOTTOM = 3

inline fun <T : View> fitterViewPosition(view: T, orientation: Int, setmargin: (T) -> Int) {
    val params = view.layoutParams as? ViewGroup.MarginLayoutParams
    when (orientation) {
        ORIENTATION_TOP -> params?.topMargin = setmargin.invoke(view)
        ORIENTATION_BOTTOM -> params?.bottomMargin = setmargin.invoke(view)
        ORIENTATION_LEFT, ORIENTATION_RIGHT -> {
            params?.leftMargin = setmargin.invoke(view)
            params?.rightMargin = setmargin.invoke(view)
        }
    }

    view.layoutParams = params
}

/**
 * 适配View的正确摆放位置，主要是margin[left,right]，不涉及View的自身宽高
 */
fun <T : View> fitterViewSingleMargin(
        view: T,
        orientation: Int,
        setLeftMargin: ((T) -> Int)? = null,
        setRightMargin: ((T) -> Int)? = null
) {
    val params = view.layoutParams as? ViewGroup.MarginLayoutParams
    when (orientation) {
        ORIENTATION_LEFT -> {
            params?.leftMargin = setLeftMargin?.invoke(view)

        }
        ORIENTATION_RIGHT -> {
            params?.rightMargin = setRightMargin?.invoke(view)
        }
    }

    view.layoutParams = params
}

/**
 * 适配 View 的正确宽高
 */
inline fun <T : View> fitterViewSize(
        view: T,
        fitterWidth: () -> Int,
        noinline fitterHeight: ((Int) -> Int)? = null
) {
    val params = view.layoutParams
    val width = fitterWidth.invoke()
    params.width = width

    val height = fitterHeight?.invoke(width)
    height?.let { params.height = it }
    view.layoutParams = params
}

fun getScreenWidth(context: Context?) = context?.resources?.displayMetrics?.widthPixels ?: 0
fun getScreenHeight(context: Context?) = context?.resources?.displayMetrics?.heightPixels ?: 0

fun dp2px(context: Context?, dp: Int) :Int = Utils.dip2px(context, dp.toFloat())

fun dp2px(context: Context?, dp: Float) = Utils.dip2px(context,dp)


fun View?.setVisible(visible: Boolean) {
    if (visible) {
        if(this?.visibility!=View.VISIBLE){
            this?.visibility = View.VISIBLE
        }
    } else {
        if(this?.visibility!=View.GONE){
            this?.visibility = View.GONE
        }
    }
}

fun getStatusBarHeight(context: Context): Int {
    // 获得状态栏高度
    val resourceId = context.resources.getIdentifier("status_bar_height", "dimen", "android")
    return context.resources.getDimensionPixelSize(resourceId)
}

fun View?.showOrGoneView(visible: Boolean) {
    this?.let {
        if(visible){
            if(it.visibility != View.VISIBLE){
                it.visibility = View.VISIBLE
            }
        }else{
            if(it.visibility != View.GONE){
                it.visibility = View.GONE
            }
        }
    }
}

fun getDrawableResource(context: Context, imgName: String): Int {
    return context.resources.getIdentifier(imgName, "drawable", context.packageName)
}

// 多个view设置可见不可见
fun setViewsVisible(visible: Boolean, vararg views: View) {
    views.forEach { view ->
        view.setVisible(visible)
    }
}

// debug测试代码
fun debugCode(action: (() -> Unit)? = null) {
    if (BuildConfig.DEBUG) {
        action?.invoke()
    }
}

//加载伸展动画
fun beginDelayedTransition(view: ViewGroup, time: Long) {
    val autoTransition = AutoTransition()
    autoTransition.duration = time
    TransitionManager.beginDelayedTransition(view, autoTransition)
}


// 分钟的转换
fun formatTimeMinute(time: Long): String {
    val lsTime = time.toInt()
    val minute = lsTime / 60 % 60
    val second = lsTime % 60

    return "${timeToString(minute)}:${timeToString(second)}"
}

fun timeToString(time: Int): String {
    return if (time < 10) {
        "0${time}"
    } else {
        time.toString()
    }
}

fun getUserName(name: String?, length: Int): String {
    if (!name.isNullOrBlank() && length > 0 && name.length > length) {
        return name.substring(0, length) + "..."
    }

    if (name.isNullOrBlank()) {
        return ""
    }

    return name
}

open class SVGACallbackImpl : SVGACallback {
    override fun onFinished() {
    }

    override fun onPause() {
    }

    override fun onRepeat() {
    }

    override fun onStep(frame: Int, percentage: Double) {
    }

}

fun ImageView.getMaskBitmap(context :Context,imageUrl: String?, mMaskedImage : Int) {
    runOnBackground {
        val mask = BitmapFactory.decodeResource(resources, mMaskedImage)
        val result: Bitmap = Bitmap.createBitmap(mask.width, mask.height, Bitmap.Config.ARGB_8888, true)
        val original :Bitmap = if(TextUtils.isEmpty(imageUrl)){
            BitmapFactory.decodeResource(resources, R.drawable.default_head)
        }else{
            GlideUtils.loadBitmapSync(context,imageUrl,mask.width,mask.height)
        }
        val tempCanvas = Canvas(result)
        val paint = Paint(Paint.ANTI_ALIAS_FLAG)
        paint.xfermode = PorterDuffXfermode(PorterDuff.Mode.DST_IN)
        tempCanvas.apply {
            drawBitmap(original, 0f, 0f, null)
            drawBitmap(mask, 0f, 0f, paint)
        }
        paint.xfermode = null

        runOnBackground(onMainThread = {
            this.apply {
                setImageBitmap(result)
                scaleType = ImageView.ScaleType.FIT_CENTER
            }
        })
    }
}

fun runOnBackground(onMainThread: () -> Unit = {}, onBackgroundThread: suspend () -> Unit = {}): Job {
    return GlobalScope.launch(Dispatchers.IO) {
        runCatching {
            onBackgroundThread()
            withContext(Dispatchers.Main) {
                onMainThread()
            }
        }.onFailure {
            it.printStackTrace()
        }
    }
}

